import warnings
from sqlalchemy import exc as sa_exc

import pytest

from workspaces.capability.services.capability_info import CapabilityInfo

pytest_plugins = ["testing.utils.conftest"]


@pytest.mark.skip("currently broken")
def test_edit_capability(mock_capability_info: CapabilityInfo):
    """
    Tests that edit_capability method properly follows its logic
    """

    starting_flush_call_count = mock_capability_info.session.flush.call_count
    mock_capability_info.session.update.return_value = 1
    # Test edit with all inputs
    assert mock_capability_info.edit_capability("null", "test", 2)
    assert mock_capability_info.session.flush.call_count == starting_flush_call_count + 1
    # Test edit with just name and steps
    assert mock_capability_info.edit_capability("null", steps="test")
    assert mock_capability_info.session.flush.call_count == starting_flush_call_count + 2
    # Test edit with just name and max_jobs
    assert mock_capability_info.edit_capability("null", max_jobs=2)
    assert mock_capability_info.session.flush.call_count == starting_flush_call_count + 3
    # Test edit with just name
    assert mock_capability_info.edit_capability("null")
    assert mock_capability_info.session.flush.call_count == starting_flush_call_count + 3
    # Test "unable to edit" condition
    mock_capability_info.session.update.return_value = 0
    assert not mock_capability_info.edit_capability("null", "test", 2)
    assert mock_capability_info.session.flush.call_count == starting_flush_call_count + 3


def test_delete_request_versions(mock_capability_info: CapabilityInfo):
    """
    Tests that delete_request_versions correctly deletes capability versions associated with a capability request
    """
    with warnings.catch_warnings():
        # suppress SQLAlchemy warnings
        warnings.simplefilter("ignore", category=sa_exc.SAWarning)
        starting_delete_call_count = mock_capability_info.session.delete.call_count
        starting_flush_call_count = mock_capability_info.session.flush.call_count
        rows_changed = mock_capability_info.delete_request_versions(1)
        assert mock_capability_info.session.delete.call_count == starting_delete_call_count + 1
        assert mock_capability_info.session.flush.call_count == starting_flush_call_count + 1


@pytest.mark.skip("This is currently broken")
def test_can_delete_capability_request(mock_capability_info: CapabilityInfo):
    """
    Tests that can_delete_capability_request correctly determines if a capability request is safe to delete
    """
    # request = mock_capability_info.create_capability_request("null")
    request = mock_capability_info.lookup_capability_request(1)

    # Request has no executions; can be deleted
    assert mock_capability_info.can_delete_capability_request(request.id) is True
    mock_capability_info.create_execution(request)
    # Request not has associated execution; can no longer be deleted
    assert mock_capability_info.can_delete_capability_request(request.id) is False
