from __future__ import annotations

import pathlib
from typing import List

from workspaces.capability.helpers_interfaces import CapabilityStepIF, ParameterIF
from workspaces.products.schema_interfaces import FutureProductIF
from workspaces.system.schema import JSONSerializable


class CapabilityIF(JSONSerializable):
    id: int
    name: str
    steps: List[CapabilityStepIF]
    max_jobs: int

    def create_request(self, parameters: ParameterIF, future_products: FutureProductIF) -> "CapabilityRequestIF":
        raise NotImplementedError


class CapabilityRequestIF(JSONSerializable):
    id: int
    state: str
    capability: CapabilityIF
    parameters: str
    future_products: str
    versions: List[CapabilityVersionIF]

    @property
    def current_version(self):
        raise NotImplementedError


class CapabilityVersionIF:
    capability_request: CapabilityRequestIF
    version_number: int
    parameters: List[ParameterIF]
    files: List[CapabilityVersionFileIF]

    def __json__(self):
        raise NotImplementedError


class CapabilityVersionFileIF:
    capability_request_id: int
    version_number: int
    filename: str
    content: bytes

    def __json__(self):
        raise NotImplementedError


class CapabilityExecutionIF:
    id: int
    capability_version: CapabilityVersionIF
    state: str
    current_step_idx: int
    current_step: CapabilityStepIF
    version: CapabilityVersionIF
    current_workflow_request_id: int
    steps: str
    capability: CapabilityIF
    capability_request: CapabilityRequestIF
    delivery_url: str
    delivery_path: pathlib.Path

    def on_last_step(self) -> bool:
        raise NotImplementedError

    def execute(self, manager: "ExecutionManagerIF"):
        raise NotImplementedError

    def __json__(self):
        raise NotImplementedError
